
'use strict';

/* Expose `compose` */
module.exports = compose;

/**
 * Compose `middleware` returning
 * a fully valid middleware comprised
 * of all those which are passed.
 *
 * This compose method is totally copied from koa-compose
 * and modified to work with the Act system. The context
 * object is removed so the next() function is all that
 * is left. Act does not claim rights to this piece of code.
 *
 * @author  KoaJS
 * @see     https://github.com/koajs/compose
 * @param   {Array} middleware
 * @return  {Function}
 */
function compose(middleware) {
  /**
   * @param   {Function} next
   * @return  {Promise}
   */
  return function nextCompositor(next) {
    /* @type {Number} */
    let index = -1;

    /* Return the first iteration */
    return dispatch(0);

    /**
     * @param  {Number} i
     * @return {Promise}
     */
    function dispatch(i) {
      /* Stop when next() is called multiple times from the same middleware */
      if (i <= index) {
        return Promise.reject(new Error('next() called multiple times'));
      }

      /* Set current index */
      index = i;

      /* Define next middleware function */
      const fn = middleware[i] || next;

      /* We are finally done */
      if (!fn) {
        return Promise.resolve();
      }

      /* Call next middleware function */
      try {
        return Promise.resolve(fn(() => dispatch(i + 1)));
      } catch (e) {
        return Promise.reject(e);
      }
    }
  };
}
